#include "scene.h"
#include "m33.h"
#include "ppc.h"

#include <iostream>

using namespace std;

Scene *scene;

Scene::Scene() {

  gui = new GUI();
  gui->show();

  int u0 = 20;
  int v0 = 50;
  int sci = 2;
  int w = sci*320;
  int h = sci*240;
  fb = new FrameBuffer(u0, v0, w, h);
  fb->label("Framebuffer");
  fb->show();
  gui->uiw->position(fb->w+20+20, v0);
  float hfov = 45.0f;
  ppc = new PPC(hfov, w, h);
  tmsN = 7;
  tms = new TMesh[tmsN];
  V3 center(0.0f, 0.0f, -175.0f);
  float sl = 30.0f;
  tms[0].SetCube(center, sl);
  tms[0].enabled = false;
  tms[1].Load("geometry/teapot1K.bin");
  AABB aabb = tms[1].GetAABB();
  float size0 = (aabb.corners[1]-aabb.corners[0]).Length();
  V3 tcenter = tms[1].GetCenter();
  tms[1].Translate(tcenter*-1.0f+center);
  float size1 = 170.0f;
  tms[1].ScaleAboutCenter(size1/size0);

  toVis = PPC(45.0f, ppc->w, ppc->h);
  toVis.C = V3(0.0f, 0.0f, -100.0f);

  tms[0].enabled = 
  tms[1].enabled = 
  tms[2].enabled = false;

  tms[3].SetRectangle(600.0f, 60.0f);
  tms[3].Translate(V3(0.0f, 0.0f, -200.0f));
  tms[3].cols[2] = 
  tms[3].cols[3] = V3(0.0f, 1.0f, 0.0f);

  tms[4].SetRectangle(600.0f, 60.0f);
  tms[4].Translate(V3(0.0f, 70.0f, -200.0f));
  tms[4].cols[2] = 
  tms[4].cols[3] = V3(0.0f, 1.0f, 0.0f);
  tms[4].msi = false;

  tms[5].SetRectangle(10.0f, 10.0f);
  tms[5].Translate(V3(0.0f, 35.0f, -200.0f));
  tms[5].cols[0] = 
  tms[5].cols[1] = 
  tms[5].cols[2] = 
  tms[5].cols[3] = V3(1.0f, 0.0f, 0.0f);

  tms[3].enabled = 
  tms[4].enabled = 
  tms[5].enabled = false;

  FrameBuffer *tmap = new FrameBuffer(u0, fb->h+v0+40, 128, 128);
  unsigned int cols[2] = {0xFF000000, 0xFF00FFFF};
  tmap->SetAsChecker(16, cols);
  tmap->show();
  tmap->label("tmap");
  tms[6].SetRectangle(60.0f, 60.0f, tmap);
  tms[6].Translate(V3(0.0f, 0.0f, -100.0f));
  tms[6].msi = false;

  Render();

}

void Scene::FrameSetup() {

  fb->Set(0xFFFFFFFF);
  fb->SetZB(0.0f);

}

void Scene::Render() {


  FrameSetup();
  for (int tmi = 0; tmi < tmsN; tmi++) {
    if (!tms[tmi].enabled)
      continue;
    tms[tmi].Render(ppc, fb);
  }

//  toVis.Visualize(ppc, fb, 10.0f, 8);

  fb->redraw();

}

void Scene::DBG() {

  tms[1].enabled = false;
  tms[2].SetFromFB(fb, ppc);
  Render();
  return;

  float visF = 10.0f;
  for (int si = 0; si < 360; si++) {
    toVis.Pan(1.0f);
    toVis.Visualize(ppc, fb, visF, 8);
    Render();
    Fl::check();
  }
  return;

  V3 center = tms[1].GetCenter();
  V3 newC(250.0f, 100.0f, 60.0f);
  V3 newVD = (center-newC).Normalized();
  V3 vinUP(0.0f, 1.0f, 0.0f);
  PPC ppc1;
  ppc->PositionAndOrient(newC, newVD, vinUP, ppc1);
  PPC ppc0(*ppc);
  int sN = 100;
  for (int si = 0; si < sN; si++) {
    float frac = (float) si / (float) sN;
    *ppc = ppc->Interpolate(&ppc0, &ppc1, frac);
    Render();
    Fl::check();
  }
  return;

  float tstep = 2.0f;
  ppc->TranslateLeft(tstep);
  Render();
  return;

  int stepsN = 360;
  V3 center1 = tms[0].GetCenter();
  for (int si = 0; si < stepsN; si++) {
    tms[0].RotateAboutArbitraryAxis(center1, V3(0.0f, 1.0f, 0.0f), 1.0f);
    Render();
    Fl::check();
  }
  return;

  float hfov = 45.0f;
  PPC ppc(hfov, fb->w, fb->h);
  
  float sl = 30.0f;
  V3 cvs[8];
  float nearz = -sl*2.0f;
  cvs[0] = V3(-sl/2.0f, sl/2.0f, nearz);
  cvs[1] = V3(-sl/2.0f, -sl/2.0f, nearz);
  cvs[2] = V3(sl/2.0f, -sl/2.0f, nearz);
  cvs[3] = V3(sl/2.0f, sl/2.0f, nearz);
  cvs[4] = V3(-sl/2.0f, sl/2.0f, nearz-sl);
  cvs[5] = V3(-sl/2.0f, -sl/2.0f, nearz-sl);
  cvs[6] = V3(sl/2.0f, -sl/2.0f, nearz-sl);
  cvs[7] = V3(sl/2.0f, sl/2.0f, nearz-sl);
  V3 projPs[8];
  V3 cols[8];
  cols[0] = cols[1] = cols[2] = cols[3] = V3(1.0f, 0.0f, 0.0f);
  cols[4] = cols[5] = cols[6] = cols[7] = V3(0.0f, 0.0f, 0.0f);
  for (int i = 0; i < 8; i++) {
    ppc.Project(cvs[i], projPs[i]);
  }
  for (int i = 0; i < 4; i++) {
    int _i = (i+1)%4;
    fb->DrawSegment(projPs[i], projPs[_i], cols[i], cols[_i]);
    fb->DrawSegment(projPs[4+i], projPs[4+_i], cols[4+i], cols[4+_i]);
    fb->DrawSegment(projPs[i], projPs[4+i], cols[i], cols[4+i]);
  }
  Render();
  return;


  return;
  V3 P(0.0f, 0.0f, -1.0f);
  V3 projP;
  bool projOK = ppc.Project(P, projP);
  return;

  M33 m;
  m[0] = V3(1.0f, 3.0f, -1.0f);
  m[1] = V3(10.0f, -31.0f, -11.0f);
  m[2] = V3(11.0f, 13.0f, 12.0f);
  M33 mi  = m.Invert();
  cerr << mi*m << endl << endl << m*mi;
  return;

  fb->Set(0xFF308080);
  Render();
  Fl::check();
  float scf = 10.5f;
  fb->Brighten(scf);
  Render();
  return;

  V3 P0(222.5f, 10.5f, 0.0f);
  V3 P1(122.5f, 410.5f, 0.0f);

  V3 c0;
  c0.SetFromColor(0xFF0000FF);
  V3 c1;
  c1.SetFromColor(0xFF00FF00);

  fb->DrawSegment(P0, P1, c0, c1);
  Render();
  return;

  unsigned int colss[2] = {0xFF000000, 0xFFFFFFFF};
  fb->SetAsChecker(32, colss);
  Render();
  return;

  for (int u = 0; u < fb->w; u++) {
    fb->Set(u, fb->h/2, 0xFF000000);
  }
  Render();
  return;

  cerr << "INFO: DBG" << endl;
  fb->Set(0xFF00FFFF);
  Render();

}

void Scene::SaveView0() {

  ppc->Save("mydbg/view0.txt");

}

void Scene::LoadView0() {

  ppc->Load("mydbg/view0.txt");
  Render();

}

void Scene::GoToView0() {

  PPC nppc;
  nppc.Load("mydbg/view0.txt");
  GoToView(&nppc);

}

void Scene::SaveView1() {

  ppc->Save("mydbg/view1.txt");

}

void Scene::LoadView1() {

  ppc->Load("mydbg/view1.txt");
  Render();

}

void Scene::GoToView1() {

  PPC nppc;
  nppc.Load("mydbg/view1.txt");
  GoToView(&nppc);

}


void Scene::GoToView(PPC *nppc) {

  PPC oppc(*ppc);
  int n = 100;
  for (int i = 0; i < n; i++) {
    float frac = (float) i / (float) (n-1);
    *ppc = ppc->Interpolate(&oppc, nppc, frac);
    Render();
    Fl::check();
  }



}
